//===------------------------------------------------------------*- C++ -*-===//
//
// T-EMU: The Terma Emulator
// (c) Terma 2015
// Authors: Mattias Holm <maho (at) terma.com>
//
//===----------------------------------------------------------------------===//

/*!
  This is the 1553 device modelling API. It provides an interface to
  be used for implementing bus controllers and remote terminals.

  The 1553 device interface is EXPERIMENTAL and UNSTABLE at the
  moment.
 */
#ifndef TEMU_MIL_STD_1553_H
#define TEMU_MIL_STD_1553_H

#ifdef __cplusplus
extern "C" {
#endif

#include "temu-c/Support/Objsys.h"

#include <assert.h>
#include <stdbool.h>
#include <stdint.h>

typedef enum temu_Mil1553Error {
  teME_NoError     = 0,
  teME_ParityError = 1,
  teME_SyncError   = 1 << 1
} temu_Mil1553Error;

typedef enum temu_Mil1553MsgType {
  teMT_BcToRt,
  teMT_RtToBc,
  teMT_RtToRtCmd,
  teMT_ModeCode
} temu_Mil1553MsgType;


typedef struct temu_Mil1553Msg {
  temu_Mil1553MsgType MsgTyp; // Pre-computed for convenience
  struct {
    uint16_t Rt:5;
    uint16_t TR:1;
    uint16_t SaMode:5;
    uint16_t CountModeCode:5;
  } Cmd;
  uint16_t *Data;
} temu_Mil1553Msg;

typedef struct temu_Mil1553DevIface {
  void (*send)(void *Device, temu_Mil1553Msg *Msg);
} temu_Mil1553DevIface;

TEMU_IFACE_REFERENCE_TYPE(temu_Mil1553Dev);


typedef struct temu_Mil1553BusIface {
  void (*connect)(void *Bus, int Subaddr, temu_Mil1553DevIfaceRef Device);
  void (*send)(void *Bus, void *Sender, temu_Mil1553Msg *Msg);
} temu_Mil1553BusIface;

TEMU_IFACE_REFERENCE_TYPE(temu_Mil1553Bus);


#ifdef __cplusplus
}
#endif


#endif /* ! TEMU_MIL_STD_1553_H */
