//===-- temu-c/Events.h - Event Queue API -----------------------*- C++ -*-===//
//
// T-EMU: The Terma Emulator
// (c) Terma 2015
// Authors: Mattias Holm <maho (at) terma.com>
//
//===----------------------------------------------------------------------===//

#ifndef TEMU_EVENTS_H
#define TEMU_EVENTS_H

#include <stdint.h>
#include "temu-c/Support/Objsys.h"

#ifdef __cplusplus
extern "C" {
#endif

#define TEMU_EVENT_NS   (1U << 0)
#define TEMU_EVENT_SYNC (1U << 8)

void* temu_allocateEventQueue(void);
void temu_disposeEventQueue(void *Queue);

int64_t temu_triggerFirstTimer(void *Queue, uint32_t Flags);
int64_t temu_triggerCurrentTimers(void *Queue, int64_t Srt, uint32_t Flags);

int64_t temu_nextTimer(void *Queue, uint32_t Flags);

void temu_postEvent(void *Queue, void (*callback)(void *Queue, void *Data),
                    void *Sender, void *Data, uint64_t Time, uint32_t Flags);

void temu_stackEvent(void *Queue, void (*callback)(void *Queue, void *Data),
                     void *Sender, void *Data, uint32_t Flags);
/*
  \brief Get the next time stamp for the given callback function.
 */
int64_t temu_eventTime(void *Queue, void (*callback)(void *Queue, void *Data),
                       void *Sender);

void temu_descheduleEvent(void *Queue,
                          void (*callback)(void *Queue, void *Data),
                          void *Sender);

typedef struct {
  void (*stackPostEvent)(void *Obj, void (*Ev)(void *, void *), void *Sender, void *Data,
                         uint32_t Flags);

  void (*postDeltaEvent)(void *Obj, void (*Ev)(void *, void *), void *Sender, void *Data,
                         int64_t Cycles, uint32_t Flags);

  void (*postAbsoluteEvent)(void *Obj, void (*Ev)(void *, void *), void *Sender, void *Data,
                            int64_t Cycles, uint32_t Flags);
  int64_t (*getEventDeltaTime)(void *Obj, void (*Ev)(void *, void *),
                               void *Sender);
  int64_t (*getEventAbsoluteTime)(void *Obj, void (*Ev)(void *, void *),
                                  void *Sender);
  void (*descheduleEvent)(void *Obj, void (*Ev)(void *, void *), void *Sender,
                          uint32_t Flags);
  void (*registerEvent)(void *Obj, const char *EvName,
                        void (*Ev)(void *, void *), uint32_t Flags);
} temu_EventIface;

TEMU_IFACE_REFERENCE_TYPE(temu_Event);

// Serialisation support
void temu_registerEvent(void *Queue, const char *Name,
                        void (*Callback)(void *Queue, void *Data),
                        uint32_t Flags);
void temu_serialiseEvents(void *Obj, void *Ctxt);
void temu_deserialiseEvents(void *Obj, void *Ctxt);


#ifdef __cplusplus
}
#endif

#endif /*  ! TEMU_EVENTS_H */
